dnl Copyright (C) 2004-2011 by Carnegie Mellon University.
dnl
dnl @OPENSOURCE_HEADER_START@
dnl
dnl Use of the SILK system and related source code is subject to the terms
dnl of the following licenses:
dnl
dnl GNU Public License (GPL) Rights pursuant to Version 2, June 1991
dnl Government Purpose License Rights (GPLR) pursuant to DFARS 252.227.7013
dnl
dnl NO WARRANTY
dnl
dnl ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER
dnl PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY
dnl PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN
dnl "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY
dnl KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT
dnl LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE,
dnl MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE
dnl OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT,
dnl SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY
dnl TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF
dnl WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES.
dnl LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF
dnl CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON
dnl CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE
dnl DELIVERABLES UNDER THIS LICENSE.
dnl
dnl Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie
dnl Mellon University, its trustees, officers, employees, and agents from
dnl all claims or demands made against them (and any related losses,
dnl expenses, or attorney's fees) arising out of, or relating to Licensee's
dnl and/or its sub licensees' negligent use or willful misuse of or
dnl negligent conduct or willful misconduct regarding the Software,
dnl facilities, or other rights or assistance granted by Carnegie Mellon
dnl University under this License, including, but not limited to, any
dnl claims of product liability, personal injury, death, damage to
dnl property, or violation of any laws or regulations.
dnl
dnl Carnegie Mellon University Software Engineering Institute authored
dnl documents are sponsored by the U.S. Department of Defense under
dnl Contract FA8721-05-C-0003. Carnegie Mellon University retains
dnl copyrights in all material produced under this contract. The U.S.
dnl Government retains a non-exclusive, royalty-free license to publish or
dnl reproduce these documents, or allow others to do so, for U.S.
dnl Government purposes only pursuant to the copyright license under the
dnl contract clause at 252.227.7013.
dnl
dnl @OPENSOURCE_HEADER_END@

dnl RCSIDENT("$SiLK: acinclude.m4 16173 2010-08-16 16:03:48Z mthomas $")


# ---------------------------------------------------------------------------
# AX_PKG_CHECK_LIBFIXBUF
#
#    Determine how to use libfixbuf, and whether libfixbuf supports
#    NetFlow v9
#
#    Output variables:  FIXBUF_CFLAGS, FIXBUF_LDFLAGS,
#                       ENABLE_IPFIX,  ENABLE_NETFLOW9
#
AC_DEFUN([AX_PKG_CHECK_LIBFIXBUF],[
    AC_SUBST(FIXBUF_CFLAGS)
    AC_SUBST(FIXBUF_LDFLAGS)

    libfixbuf_required_version="$1"
    libfixbuf_netflow9_supported="$2"

    ENABLE_IPFIX=0
    ENABLE_NETFLOW9=0

    m4_ifdef([PKG_CHECK_MODULES], [AX_PKG_CHECK_LIBFIXBUF_PKGCONFIG])

    if test "x$ENABLE_IPFIX" = "x1"
    then
        # Cache current values
        sk_save_LDFLAGS="$LDFLAGS"
        sk_save_LIBS="$LIBS"
        sk_save_CFLAGS="$CFLAGS"
        sk_save_CPPFLAGS="$CPPFLAGS"

        FIXBUF_LDFLAGS="$LIBFIXBUF_LIBS"
        LIBS="$FIXBUF_LDFLAGS $LIBS"

        FIXBUF_CFLAGS="$LIBFIXBUF_CFLAGS"
        CFLAGS="$FIXBUF_CFLAGS $CFLAGS"

        AC_MSG_CHECKING([usability of libfixbuf library and headers])
        AC_LINK_IFELSE(
            [AC_LANG_PROGRAM([
#include <fixbuf/public.h>
                ],[
fbInfoModel_t *m = fbInfoModelAlloc();
                 ])],[
            AC_MSG_RESULT([yes])
            ENABLE_IPFIX=1
            ],[
            AC_MSG_RESULT([no])
            ENABLE_IPFIX=0])

dnl 2010-Apr-28.  We now enable NetFlow v9 by using pkg-config to
dnl               check for libfixbuf 1.0.0.
dnl
dnl         if test "x$ENABLE_IPFIX" = "x1"
dnl         then
dnl             AC_MSG_CHECKING([whether libfixbuf supports NetFlow v9])
dnl             AC_LINK_IFELSE(
dnl                 [AC_LANG_PROGRAM([
dnl #include <fixbuf/public.h>
dnl                     ],[
dnl fbCollector_t *collector = NULL;
dnl GError *err = NULL;
dnl fbCollectorSetNetflowV9Translator(collector, &err);
dnl                      ])],[
dnl                 AC_MSG_RESULT([yes])
dnl                 ENABLE_NETFLOW9=1
dnl                 ],[
dnl                 AC_MSG_RESULT([no])
dnl                 ENABLE_NETFLOW9=0])
dnl         fi

        # Restore cached values
        LDFLAGS="$sk_save_LDFLAGS"
        LIBS="$sk_save_LIBS"
        CFLAGS="$sk_save_CFLAGS"
        CPPFLAGS="$sk_save_CPPFLAGS"
    fi

    if test "x$ENABLE_IPFIX" = "x0"
    then
        FIXBUF_LDFLAGS=
        FIXBUF_CFLAGS=
    fi

    AM_CONDITIONAL(HAVE_FIXBUF, [test "x$ENABLE_IPFIX" = "x1"])

    AC_DEFINE_UNQUOTED([ENABLE_IPFIX], [$ENABLE_IPFIX],
        [Define to 1 to build with support for IPFIX.  Define to 0 otherwise.
         Requires libfixbuf support and the <fixbuf/public.h> header file.])

    AC_DEFINE_UNQUOTED([ENABLE_NETFLOW9], [$ENABLE_NETFLOW9],
        [Define to 1 to build with support for NetFlow v9.  Define to 0
         otherwise.  Requires libfixbuf-1.0 support and the
         <fixbuf/public.h> header file.  (Expert users may choose to
         enable this when using libfixbuf-0.8.0 or later, though that
         version of libfixbuf does not properly handle NetFlow v9 as
         generated by CISCO routers.)])

])# AX_PKG_CHECK_LIBFIXBUF


# ---------------------------------------------------------------------------
# AX_PKG_CHECK_LIBFIXBUF_PKGCONFIG
#
#    Run the part of --with-fixbuf that requires pkgconfig
#
AC_DEFUN([AX_PKG_CHECK_LIBFIXBUF_PKGCONFIG],[
    sk_pkg_config=""
    AC_ARG_WITH([libfixbuf],[AS_HELP_STRING([--with-libfixbuf=DIR],
            [find libfixbuf package configuration file (libfixbuf.pc) in the directory DIR (prepends DIR to the PKG_CONFIG_PATH)])[]dnl
        ],[
            if test "x$withval" != "xyes"
            then
                sk_pkg_config="$withval"
            fi
    ])

    if test "x$sk_pkg_config" != "xno"
    then
        if test "x$sk_pkg_config" != "x"
        then
            sk_save_PKG_CONFIG_PATH="$PKG_CONFIG_PATH"
            PKG_CONFIG_PATH="$sk_pkg_config:$PKG_CONFIG_PATH"
            export PKG_CONFIG_PATH
        fi

        # first see if we have libfixbuf at all
        PKG_CHECK_MODULES([LIBFIXBUF],
            [libfixbuf >= $libfixbuf_required_version],
            [ENABLE_IPFIX=1],[ENABLE_IPFIX=0])

        # if we have it, does it support NetFlow v9
        if test "x$ENABLE_IPFIX" = "x1"
        then
            m4_ifdef([PKG_CHECK_EXISTS],
                     [PKG_CHECK_EXISTS(
                          [libfixbuf >= $libfixbuf_netflow9_supported],
                          [ENABLE_NETFLOW9=1], [ENABLE_NETFLOW9=0])],
                     [PKG_CHECK_MODULES([LIBNETFLOWV9],
                          [libfixbuf >= $libfixbuf_netflow9_supported],
                          [ENABLE_NETFLOW9=1], [ENABLE_NETFLOW9=0])])
        fi

        if test "x$sk_pkg_config" != "x"
        then
            PKG_CONFIG_PATH="$sk_save_PKG_CONFIG_PATH"
            export PKG_CONFIG_PATH
        fi
    fi
])# AX_PKG_CHECK_LIBFIXBUF_PKGCONFIG

dnl Local Variables:
dnl mode:autoconf
dnl indent-tabs-mode:nil
dnl End:
