#! /usr/bin/perl -w
#
#
# RCSIDENT("$SiLK: rwflowpack-pack-respool.pl 16270 2010-09-09 21:30:31Z mthomas $")

use strict;
use SiLKTests;
use File::Find;

my $rwflowpack = $ENV{RWFLOWPACK} || './rwflowpack';

# find the apps we need.  this will exit 77 if they're not available
my $rwcat   = check_silk_app('rwcat');
my $rwsplit = check_silk_app('rwsplit');
my $rwuniq  = check_silk_app('rwuniq');

# find the data files we use as sources, or exit 77
my %file;
$file{data} = get_data_or_exit77('data');

# prefix any existing PYTHONPATH with the proper directories
check_python_bin();

# set the environment variables required for rwflowpack to find its
# packing logic plug-in
add_plugin_dirs('/site/twoway');

# create our tempdir.  clean up unless the appropriate environment
# variable is set.
my $tmpdir = File::Temp::tempdir(CLEANUP => !$ENV{SK_TESTS_SAVEOUTPUT});
exit 77 unless -d $tmpdir;

# Generate the data files
my $cmd = ("$rwsplit --basename=$tmpdir/rwsplit-out --byte-limit=1200000000 "
           .$file{data});
unless (check_exit_status($cmd)) {
    die "ERROR: $rwsplit exited with error\n";
}
my @input_files = glob("$tmpdir/rwsplit-out.*");

# the command that wraps rwflowpack
$cmd = join " ", ("$SiLKTests::PYTHON $srcdir/tests/rwflowpack-daemon.py",
                  ($ENV{SK_TESTS_VERBOSE} ? "--verbose" : ()),
                  #"--log-level=debug",
                  (map {"--move $_:incoming"} @input_files),
                  "--limit=501876",
                  "--basedir=$tmpdir",
                  "--",
                  "--input-mode=respool",
                  "--incoming-directory=$tmpdir/incoming",
                  "--polling-interval=5",
                  "--flat-archive",
    );

# run it and check the MD5 hash of its output
check_md5_output('a78a286719574389a972724d761c931e', $cmd);

# the following directories should be empty
my @empty_dirs = qw(error incoming incremental sender);
for my $d (map {"$tmpdir/$_"} @empty_dirs) {
    next unless -d $d;
    unless (opendir D, $d) {
        warn "ERROR: Cannot open directory '$d': $!\n";
        next;
    }
    while (my $f = readdir(D)) {
        next if $f =~ /^\.\.?$/;
        die "ERROR: Found unexpected file '$d/$f'\n";
    }
    closedir D;
}

# input files should now be in the archive directory
my $archive_dir = "$tmpdir/archive/";
unless (opendir D, $archive_dir) {
    die "ERROR: Cannot open directory '$archive_dir': $!\n";
}
my %expected;
for my $f (@input_files) {
    die unless $f =~ m,.*/(.+),;
    $expected{$1} = "$archive_dir/$1";
}
while (my $f = readdir(D)) {
    next if $f =~ /^\.\.?$/;
    if ($expected{$f}) {
        delete $expected{$f};
    }
    else {
        die "ERROR: Found unexpected file '$archive_dir/$f'\n";
    }
}
if (keys %expected) {
    die "ERROR: Missing file '", join("', '", values %expected), "'\n";
}
closedir D;

# path to the data directory
my $data_dir = "$tmpdir/root";
die "ERROR: Missing data directory '$data_dir'\n"
    unless -d $data_dir;

# check the output
$cmd = ("find $data_dir -type f -print"
        ." | $rwcat --xargs"
        ." | $rwuniq --ipv6=ignore --fields=sip,sensor,type,stime"
        ." --values=records,packets,stime,etime --sort");
check_md5_output('f961d91d6c5b8e453ce60545ab856623', $cmd);

# successful!
exit 0;
